#include <Adafruit_NeoPixel.h>

Adafruit_NeoPixel pixels(16, 9, NEO_GRB + NEO_KHZ800);


const int fr = A4; //первое реле
const int sr = 0;  // второе реле
const int b = 10;  //кнопка
const int ph = A3; //датчик

//количество делителей напряжения
const int numberOfDividers = 3;

struct point{
  int x;
  int y;
};

//функция высчитывает тангенс угла наклона прямой по 2 точкам
float getK(point p1, point p2){
  return float((p2.y - p1.y))/float((p2.x -p1.x));
}

//Массив напряжений в миллиамперах с делителей
//После спайки делителей надо замерить напряжение на них и занести в массив
// Последнее значение 0, так как проверяем землей 
int mas[3] = {55, 713, 0};

//получаем analogRead() c земли
int getDataFromZero(){
  digitalWrite(fr, HIGH);
  digitalWrite(sr, HIGH);
  delay(1000);
  return analogRead(ph);
}

//получаем analogRead() c первого делителя
int getDataFromFirstD(){
  digitalWrite(fr, LOW);
  digitalWrite(sr, HIGH);
  delay(1000);
  return analogRead(ph);
}

//получаем analogRead() c второго делителя
int getDataFromSecD(){
  digitalWrite(fr, HIGH);
  digitalWrite(sr, LOW);
  delay(1000);
  return analogRead(ph);
}

//создаём массив из точек на графике зависимости напряжения на делителях и полученных данных
point* readingData(){
  point* result;
  result = new point[numberOfDividers];
  result[0].x = mas[0];
  result[1].x = mas[1];
  result[2].x = mas[2];
  result[0].y = getDataFromFirstD();
  result[1].y = getDataFromSecD();
  result[2].y = getDataFromZero();
  return result;
}

//функция наличия данных в промежутке
bool isInGap(float fInput, float sInput, float tInput, float gap = 0.05){
  return (abs(fInput-sInput) <= gap)&&(abs(fInput-tInput) <= gap)&&(abs(tInput-sInput) <= gap);
}

bool isInGap(int fInput, int sInput, int gap = 5){
  return (abs(fInput-sInput) <= gap);
}

//тест датчика
int testPH(){
  // получаем график
  point* data = readingData();
/* 
  Serial.print(getK(data[0], data[2]));
  Serial.print(' ');
  Serial.print(getK(data[1], data[2]));
  Serial.print(' ');
  Serial.print(getK(data[0], data[1]));
  Serial.print("     ");
  Serial.println(data[2].y);
*/

  //проверяем нулевую точку графика. Она нулевая при любом потенциометре
  if(!isInGap(data[2].y, 404)){
    return 1;
  }
  // проверяем коэффициент наклона прямой между тремя точками
  if(!isInGap(getK(data[0], data[2]), getK(data[1], data[2]), getK(data[0], data[1]))){
    return 2; 
  }

  if(getK(data[0], data[2]) < -0.6 || getK(data[0], data[2]) > -0.3 ){
    return 3;
}
return 0;
}

// -2 - бегущий огонек
// -1 - не светит
// 0 - зеленый
// x > 0 - красный с x яркими
void error(char e) {
  pixels.clear();
  switch (e) {
    case -2:
      for (byte i = 0; i < 16; i++) {
        pixels.clear();
        pixels.setPixelColor(i, pixels.Color(3, 3, 3));
        pixels.show();
        delay(35);
      }
      break;
    case -1: break;
    case 0:
      for (byte i = 0; i < 16; i++)
        pixels.setPixelColor(i, pixels.Color(0, 3, 0));
      break;
    default:
      for (byte i = 0; i < 16; i++)
        pixels.setPixelColor(i, pixels.Color(3, 0, 0));

      for (byte i = 0; i < e; i++)
        pixels.setPixelColor(i, pixels.Color(100, 0, 0));
      break;
  }
  pixels.show();
}


void setup() {
  pinMode(ph, INPUT);
  pinMode(fr, OUTPUT);
  pinMode(sr, OUTPUT);
  pinMode(b, INPUT);
  digitalWrite(fr, LOW);
  digitalWrite(sr, LOW);
  pixels.begin();
  error(-2); // по экрану пробежит белая точка
  error(-1); // потом погасим экран

  //Serial.begin(9600);
}

void loop() {
  if (digitalRead(b) == 1) {
    while (digitalRead(b) == 1); // сработает по отпусканию кнопки

    error(-1); // черный
    auto e = testPH();
    // 1 огонек - ноль смещен
    // 2 огонька - различный K
    // 3 огонька - К за границами доступного диапазона   
    if (e > 0) {
      error(e);
      return;
    }
    error(0); // все хорошо - зеленый
  }

  delay(20);
  
}
